@@ -102,11 +102,20 @@ class Distance(SessionMapping):
     def __invert__(self):
         return 1.0 - self.distance
 
+    @staticmethod
+    def make_dummy(session):
+        return Distance(
+            session,
+            {key: 0.0 for key in session._mask.keys()}
+        )
+
+
 ###########################################################################
 #                             Import Aliases                              #
 ###########################################################################
 
 from munin.distance.bpm import BPMDistance
+from munin.distance.date import DateDistance
 from munin.distance.genre import GenreTreeDistance, GenreTreeAvgLinkDistance
 from munin.distance.rating import RatingDistance
 from munin.distance.moodbar import MoodbarDistance
