@@ -43,7 +43,6 @@ Reference
 """
 
 # Stdlib:
-import urllib.request
 import pickle
 import shelve
 import json
@@ -51,10 +50,13 @@ import re
 
 from urllib.request import urlopen
 from urllib.parse import quote
+from collections import Counter
 
 # Internal imports:
 from munin.provider import Provider
 from munin.session import get_cache_path, check_or_mkdir
+from munin.generate_genre_list import load_genrelist
+
 
 from munin.provider.normalize import \
     ArtistNormalizeProvider, \
@@ -71,26 +73,6 @@ from pyxdameraulevenshtein import \
     levenshtein
 
 
-def load_genrelist_from_echonest(dump_path=None):
-    """Try yo load a list of genres from echonest (with libglyr's APIKEY)
-
-    This requires a working internet connection obviously.
-
-    :param dump_path: Pickle the list under this path.
-    :type dump_path: string
-    :returns: a list with ~700 genres.
-    """
-    URL = 'http://developer.echonest.com/api/v4/artist/list_genres?api_key=ZSIUEIVVZGJVJVWIS&format=json'
-    json_file = json.loads(urllib.request.urlopen(URL).read().decode('utf-8'))
-    genres = [pair['name'] for pair in json_file['response']['genres']]
-
-    # Pickle the list if desired:
-    if dump_path is not None:
-        with open(dump_path, 'wb') as f:
-            pickle.dump(genres, f)
-    return genres
-
-
 class Tree:
     """
     Modelling a Tree of Genres. Each Tree can be a Node (no children) or
@@ -224,12 +206,7 @@ def build_genre_tree():
     :returns: The root node of the Genre tree.
     """
     # Add a caching layer:
-    dump_path = get_cache_path('genre_list.dump')
-    try:
-        with open(dump_path, 'rb') as f:
-            genres = pickle.load(f)
-    except (OSError, IOError, AttributeError):
-        genres = load_genrelist_from_echonest(dump_path)
+    genres = load_genrelist()
 
     # Create the root node ('music' is always optional)
     root = Tree('music', children=[Tree(genre) for genre in genres])
@@ -337,7 +314,7 @@ def build_genre_path_best_of_two(root, words):
     fst_try = build_genre_path_single(root, words)
     words.reverse()
     snd_try = build_genre_path_single(root, words)
-    return sorted(fst_try + snd_try)
+    return sorted(set(fst_try + snd_try))
 
 
 def build_genre_path_all(root, words):
@@ -409,6 +386,22 @@ def load_genre_tree(pickle_path):
 DISCOGS_API_SEARCH_URL = "http://api.discogs.com/database/search?type=release&q={artist}"
 
 
+def _filter_crosslinks(genre_set, style_set):
+    for key in genre_set:
+        if key in style_set:
+            del style_set[key]
+
+
+def _filter_spam(counter):
+    if not counter:
+        return
+
+    avg = sum(counter.values()) // len(counter)
+    for key in list(counter):
+        if counter[key] < avg:
+            del counter[key]
+
+
 def _find_right_genre(json_doc, artist, album, persist_on_album):
     """
     Try to read the correct genre from the json document by discogs.
@@ -418,7 +411,7 @@ def _find_right_genre(json_doc, artist, album, persist_on_album):
     :param persist_on_album: If False, other albums of this artist are valid sources too.
     :returns: A set of music genres (i.e. rock) and a set of styles (i.e. death metal)
     """
-    genre_set, style_set = set(), set()
+    genre_set, style_set = Counter(), Counter()
     for item in json_doc['results']:
         # Some artist items have not a style in them.
         # Skip these items.
@@ -434,17 +427,20 @@ def _find_right_genre(json_doc, artist, album, persist_on_album):
 
         # Try to outweight spelling errors, or small
         # pre/suffixes to the artist. (i.e. 'the beatles' <-> beatles')
-        if levenshtein(artist, remote_artist) > 0.75:
+        if levenshtein(artist, remote_artist) > 0.5:
             continue
 
         # Same for the album:
-        if persist_on_album and levenshtein(album, remote_album) > 0.75:
+        if persist_on_album and levenshtein(album, remote_album) > 0.5:
             continue
 
         # Remember the set of all genres and styles.
         genre_set.update(item['genre'])
         style_set.update(item['style'])
 
+    _filter_spam(genre_set)
+    _filter_spam(style_set)
+    _filter_crosslinks(genre_set, style_set)
     return genre_set, style_set
 
 
@@ -504,8 +500,11 @@ def find_genre_via_discogs(artist, album):
 
     # Bulid a genre string that is formatted this way:
     #  genre1; genre2 [;...] / style1, style2, style3 [,...]
-    #  blues; rock / blues rock, country rock, christian blues
-    return ' / '.join(('; '.join(genre_set), ', '.join(style_set)))
+    #  blues, rock / blues rock, country rock, christian blues
+    return ' / '.join((
+        ', '.join(k for k, v in genre_set.most_common(3)),
+        ', '.join(k for k, v in style_set.most_common(4))
+    ))
 
 
 class DiscogsGenreProvider(Provider):
@@ -683,7 +682,7 @@ if __name__ == '__main__':
             print('Prepared Words:', words)
             print('Result Path   :', paths)
             if paths:
-                print('Resolved      :', root.resolve_path(paths[0]))
+                print('Resolved      :', [root.resolve_path(path) for path in paths])
             print('---------------')
 
         # Silly graph drawing playground ahead:
